@@ -1,11 +1,12 @@
+use swc_common::{BytePos, Span};
+use swc_css_ast::*;
+
 use super::{input::ParserInput, PResult, Parser};
 use crate::{
     error::{Error, ErrorKind},
     parser::{Ctx, RuleContext},
     Parse,
 };
-use swc_common::{BytePos, Span};
-use swc_css_ast::*;
 
 #[derive(Debug, Default)]
 pub(super) struct AtRuleContext {}
@@ -608,11 +609,14 @@ where
 
                 Ok(KeyframeSelector::Ident(ident))
             }
-            tok!("percent") => Ok(KeyframeSelector::Percent(self.parse()?)),
+            tok!("percentage") => Ok(KeyframeSelector::Percentage(self.parse()?)),
             _ => {
                 let span = self.input.cur_span()?;
 
-                Err(Error::new(span, ErrorKind::Expected("ident or percentage")))
+                return Err(Error::new(
+                    span,
+                    ErrorKind::Expected("ident or percentage token"),
+                ));
             }
         }
     }
