@@ -1,4 +1,4 @@
-use swc_common::{BytePos, Spanned};
+use swc_common::BytePos;
 use swc_css_ast::*;
 
 use super::{input::ParserInput, Ctx, PResult, Parser};
@@ -14,80 +14,9 @@ impl<I> Parser<I>
 where
     I: ParserInput,
 {
-    /// Ported from `parseDeclaration` of esbuild.
-    ///
-    /// https://github.com/evanw/esbuild/blob/a9456dfbf08ab50607952eefb85f2418968c124c/internal/css_parser/css_parser.go#L987
-    ///
-    /// Returned [BytePos] is `hi`.
-    pub(super) fn parse_property_values(&mut self) -> PResult<(Vec<Value>, BytePos)> {
-        let start = self.input.state();
-        let mut values = vec![];
-        let mut state = self.input.state();
-        let start_pos = self.input.cur_span()?.lo;
-        let mut hi = self.input.last_pos()?;
-
-        loop {
-            if is_one_of!(self, EOF, ";", "}", "!", ")", "]") {
-                self.input.reset(&state);
-                break;
-            }
-
-            let v = self.parse_one_value_inner()?;
-
-            hi = v.span().hi;
-            values.push(v);
-            state = self.input.state();
-
-            if !eat!(self, " ")
-                && !is_one_of!(
-                    self,
-                    ",",
-                    "/",
-                    "function",
-                    "ident",
-                    "dimension",
-                    "percentage",
-                    "num",
-                    "str",
-                    "#",
-                    "url",
-                    "[",
-                    "{",
-                    "("
-                )
-            {
-                if self.ctx.recover_from_property_value
-                    && !is_one_of!(self, EOF, ";", "}", "!", ")", "]")
-                {
-                    self.input.reset(&start);
-
-                    let mut tokens = vec![];
-                    while !is_one_of!(self, EOF, ";", "}", "!", ")", "]") {
-                        tokens.extend(self.input.bump()?);
-                    }
-
-                    let span = span!(self, start_pos);
-                    let v = Value::Tokens(Tokens { span, tokens });
-
-                    self.errors
-                        .push(Error::new(span, ErrorKind::InvalidDeclarationValue));
-
-                    return Ok((vec![v], hi));
-                }
-
-                break;
-            }
-        }
-
-        // TODO: Make this lazy
-        Ok((values, hi))
-    }
-
     /// Parse value as <declaration-value>.
-    pub(super) fn parse_declaration_value(&mut self) -> PResult<Tokens> {
-        let start = self.input.cur_span()?.lo;
-
-        let mut tokens = vec![];
+    pub(super) fn parse_declaration_value(&mut self) -> PResult<Vec<Value>> {
+        let mut value = vec![];
         let mut balance_stack: Vec<Option<char>> = vec![];
 
         // The <declaration-value> production matches any sequence of one or more
@@ -153,21 +82,16 @@ where
             let token = self.input.bump()?;
 
             match token {
-                Some(token) => tokens.push(token),
+                Some(token) => value.push(Value::PreservedToken(token)),
                 None => break,
             }
         }
 
-        Ok(Tokens {
-            span: span!(self, start),
-            tokens,
-        })
+        Ok(value)
     }
 
     /// Parse value as <any-value>.
-    pub(super) fn parse_any_value(&mut self) -> PResult<Tokens> {
-        let start = self.input.cur_span()?.lo;
-
+    pub(super) fn parse_any_value(&mut self) -> PResult<Vec<TokenAndSpan>> {
         let mut tokens = vec![];
         let mut balance_stack: Vec<Option<char>> = vec![];
 
@@ -225,13 +149,10 @@ where
             }
         }
 
-        Ok(Tokens {
-            span: span!(self, start),
-            tokens,
-        })
+        Ok(tokens)
     }
 
-    fn parse_one_value_inner(&mut self) -> PResult<Value> {
+    pub(super) fn parse_one_value_inner(&mut self) -> PResult<Value> {
         // TODO remove me
         self.input.skip_ws()?;
 
@@ -270,7 +191,14 @@ where
 
             tok!("percentage") | tok!("dimension") | tok!("num") => {
                 let span = self.input.cur_span()?;
-                let base = self.parse_basical_numeric_value()?;
+                let base = match cur!(self) {
+                    tok!("percentage") => Value::Percentage(self.parse()?),
+                    tok!("dimension") => Value::Dimension(self.parse()?),
+                    tok!("num") => Value::Number(self.parse()?),
+                    _ => {
+                        unreachable!()
+                    }
+                };
 
                 return self.parse_numeric_value_with_base(span.lo, base);
             }
@@ -287,27 +215,17 @@ where
                 return Ok(Value::Ident(self.parse()?));
             }
 
-            tok!("[") => return self.parse_square_brackets_value().map(From::from),
+            tok!("[") => return Ok(Value::SimpleBlock(self.parse()?)),
 
-            tok!("(") => return self.parse_round_brackets_value().map(From::from),
+            tok!("(") => return Ok(Value::SimpleBlock(self.parse()?)),
 
-            tok!("{") => {
-                return self.parse_brace_value().map(From::from);
-            }
+            tok!("{") => return Ok(Value::SimpleBlock(self.parse()?)),
 
             tok!("#") => return Ok(Value::Color(Color::HexColor(self.parse()?))),
 
             _ => {}
         }
 
-        if is_one_of!(self, "<!--", "-->", "!", ";") {
-            let token = self.input.bump()?.unwrap();
-            return Ok(Value::Tokens(Tokens {
-                span,
-                tokens: vec![token],
-            }));
-        }
-
         Err(Error::new(span, ErrorKind::Expected("Declaration value")))
     }
 
@@ -352,105 +270,6 @@ where
         Ok(base)
     }
 
-    fn parse_brace_value(&mut self) -> PResult<SimpleBlock> {
-        let span = self.input.cur_span()?;
-
-        expect!(self, "{");
-
-        let brace_start = self.input.cur_span()?.lo;
-        let mut tokens = vec![];
-
-        let mut brace_cnt = 1;
-        loop {
-            if is!(self, "}") {
-                brace_cnt -= 1;
-                if brace_cnt == 0 {
-                    break;
-                }
-            }
-            if is!(self, "{") {
-                brace_cnt += 1;
-            }
-
-            let token = self.input.bump()?;
-            match token {
-                Some(token) => tokens.push(token),
-                None => break,
-            }
-        }
-
-        let brace_span = span!(self, brace_start);
-        expect!(self, "}");
-
-        Ok(SimpleBlock {
-            span: span!(self, span.lo),
-            name: '{',
-            // TODO refactor me
-            value: vec![Value::Tokens(Tokens {
-                span: brace_span,
-                tokens,
-            })],
-        })
-    }
-
-    fn parse_basical_numeric_value(&mut self) -> PResult<Value> {
-        match cur!(self) {
-            tok!("percentage") => Ok(Value::Percentage(self.parse()?)),
-            tok!("dimension") => Ok(Value::Dimension(self.parse()?)),
-            tok!("num") => Ok(Value::Number(self.parse()?)),
-            _ => {
-                unreachable!()
-            }
-        }
-    }
-
-    fn parse_square_brackets_value(&mut self) -> PResult<SimpleBlock> {
-        let span = self.input.cur_span()?;
-
-        expect!(self, "[");
-
-        self.input.skip_ws()?;
-
-        let value = self.parse_property_values()?.0;
-
-        self.input.skip_ws()?;
-
-        expect!(self, "]");
-
-        Ok(SimpleBlock {
-            span: span!(self, span.lo),
-            name: '[',
-            value,
-        })
-    }
-
-    fn parse_round_brackets_value(&mut self) -> PResult<SimpleBlock> {
-        let span = self.input.cur_span()?;
-
-        expect!(self, "(");
-
-        self.input.skip_ws()?;
-
-        let value = if is!(self, ")") {
-            vec![]
-        } else {
-            let ctx = Ctx {
-                allow_operation_in_value: true,
-                ..self.ctx
-            };
-
-            self.with_ctx(ctx).parse_property_values()?.0
-        };
-
-        expect!(self, ")");
-
-        Ok(SimpleBlock {
-            span: span!(self, span.lo),
-            name: '(',
-            value,
-        })
-    }
-
     pub fn parse_simple_block(&mut self, ending: char) -> PResult<SimpleBlock> {
         let start_pos = self.input.last_pos()? - BytePos(1);
         let mut simple_block = SimpleBlock {
@@ -495,13 +314,9 @@ where
                     return Ok(simple_block);
                 }
                 _ => {
-                    let span = self.input.cur_span()?;
                     let token = self.input.bump()?.unwrap();
 
-                    simple_block.value.push(Value::Tokens(Tokens {
-                        span: span!(self, span.lo),
-                        tokens: vec![token],
-                    }));
+                    simple_block.value.push(Value::PreservedToken(token));
                 }
             }
         }
@@ -514,14 +329,10 @@ where
             tok!("{") => Ok(Value::SimpleBlock(self.parse_simple_block('}')?)),
             tok!("function") => Ok(Value::Function(self.parse()?)),
             _ => {
-                let span = self.input.cur_span()?;
                 let token = self.input.bump()?;
 
                 match token {
-                    Some(t) => Ok(Value::Tokens(Tokens {
-                        span: span!(self, span.lo),
-                        tokens: vec![t],
-                    })),
+                    Some(t) => Ok(Value::PreservedToken(t)),
                     _ => {
                         unreachable!();
                     }
